﻿using System;
using System.Text;

using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Runtime;

[assembly: CommandClass(typeof(TestPrj.cMyCommands))]

namespace TestPrj
{

   public class cMyCommands
   {
      [CommandMethod("TestPrj_CMD01")]
      public static void TestPrj_CMD01()
      {
         Document tAcadDoc = Application.DocumentManager.MdiActiveDocument;
         Transaction tTrAct = null;

         try
         {
            tTrAct = tAcadDoc.TransactionManager.StartTransaction();

            PromptEntityResult tSelRes =tAcadDoc.Editor.GetEntity(@"Bemassung wählen: ");
            if (tSelRes.Status == PromptStatus.OK)
            {
               Entity tEnt = (Entity)tTrAct.GetObject(tSelRes.ObjectId, OpenMode.ForRead);
               string DimTxtExp = GetDimTextExponent(tEnt);
            }

            tTrAct.Commit();
         }
         catch 
         {}
         finally
         {
            tTrAct.Dispose();
         }



         Application.DocumentManager.MdiActiveDocument.Editor.Regen();
      }


      public static string GetDimTextExponent(Entity SelectedEnt)
      {
         Dimension dim;
         Type dimType = SelectedEnt.GetType();

         // Es werden nur Längenmaße überschrieben
         if ((dimType == typeof(LineAngularDimension2)) | (dimType == typeof(OrdinateDimension)) | (dimType == typeof(Point3AngularDimension)))
         {
            return null;
         }
         else
         {
            dim = (Dimension)SelectedEnt;
         }

         // Wenn keine Nachkommastellen vorhanden sind, können wir auch abbrechen
         if (dim.Dimdec == 0)
            return null;

         string dimText = dim.DimensionText;

         // Ist der Bemaßungstext überschrieben und beinhaltet kein Maß, können wir abbrechen
         if (dimText != "")
         {
            if (!dimText.Contains("<>"))
               return null;
         }


         double dimMeasurement = dim.Measurement;
         double dimRound = dim.Dimrnd;
         int dimDec = dim.Dimdec;

         if (dimRound != 0)
            //TODO das musste ich aendern, da ich keine myUtils-Klasse habe :-)
            dimMeasurement = Math.Round(dimMeasurement,2);

         // Wir setzten den Text für die Bemaßungslänge
         string dimTextMeasurement = dimMeasurement.ToString("F" + dimDec.ToString());

         // Wenn die Zeichnung in Metern un der Dimsclae 1 ist, das Maß ebenfalls kleiner als 1 ist, ...
         if (HostApplicationServices.WorkingDatabase.Insunits == UnitsValue.Meters & dim.Dimscale == 1 & dimMeasurement < 1)
         {
            // ... wandeln wir in cm um und reduzieren die Nachkommastellen.
            dimDec -= 2;
            dimTextMeasurement = (dimMeasurement * 100).ToString("F" + dimDec.ToString());
         }

         StringBuilder sbDim = new StringBuilder(dimTextMeasurement);

         if (dimTextMeasurement.EndsWith("0"))
         {
            if (dimDec < 2)
               sbDim.Remove(sbDim.Length - 2, 2);
            else
               sbDim.Remove(sbDim.Length - 1, 1);
         }
         else
         {
            if (dimDec < 2)
               sbDim.Remove(sbDim.Length - 2, 1);

            sbDim.Insert(sbDim.Length - 1, @"{\A1;\H0.7x;\S");
            sbDim.Insert(sbDim.Length, @"^;}");
         }

         if (dim.Prefix != "")
            sbDim.Insert(0, dim.Prefix);
         if (dim.Suffix != "")
            sbDim.Insert(sbDim.Length, dim.Suffix);
         if (dimText != "")
         {
            int Wo = dimText.IndexOf("<>");
            if (Wo > 0)
               sbDim.Insert(0, dimText.Substring(0, Wo));
            if (Wo < dimText.Length - 2)
               sbDim.Insert(sbDim.Length, dimText.Substring(Wo + 2));
         }

         return sbDim.ToString();
      }
   }
}
